
# This file is part of FatxFast.
#
#    FatxFast is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    FatxFast is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with FatxFast.  If not, see <http://www.gnu.org/licenses/>.

# Versioning based on: 
# http://en.wikipedia.org/wiki/Versioning#Designating_development_stage
__author__ = "dryatu (c) 2013"
__version__ = "1.2.5"

from FatxFast.tilemap.tiledtmxloader import tmxreader
from FatxFast.tilemap.editor.resourceloader import ResourceLoader
from FatxFast.tilemap.editor import tile
from FatxFast.tilemap.editor import layer
from FatxFast.resource import resource_manager
import pygame

class WorldEditor(object):
    
    def __init__(self):
        self.resources = ResourceLoader()
        # Map data container
        self.tile_map = tmxreader.TileMap()
        # Current layer of operation
        self._layer = 0

    def add_layer(self, level, layer):
        if level:
            self.tile_map.layers.insert(level, layer)
        else:
            self.tile_map.layers.append(layer)
            layer._level = len(self.tile_map.layers)-1

    def add_object(self, obj):
        self.tile_map.layers[layer.OBJECT_LAYER].objects.append(obj)

    def create_layer(self, level=0, **kwargs):
        # Creates a layer of the map size
        if "default_image" in kwargs:
            image = self.resources.indexed_tiles[kwargs["default_image"]][2]
            kwargs["default_image"] = image

        lay = layer.Layer(num_tiles_x=self.tile_map.width, 
            num_tiles_y=self.tile_map.height, **kwargs)
        self.add_layer(level, lay)

    def create_object(self, x, y, width, height, type, **properties):
        if self._layer is layer.OBJECT_LAYER:
            obj = tmxreader.MapObject()
            obj.x, obj.y, obj.width, obj.height = x, y, width, height
            obj.type = type
            obj.properties = properties
            self.add_object(obj)

    def create_object_layer(self, level=0, **kwargs): 
        layer = tmxreader.MapObjectGroupLayer()
        self.add_layer(level, layer)

    def call_action_in_a_region(self, action, param, x, y, ex=None, ey=None, 
        **kwargs):
        for _y in xrange(y, ey+1):
            for _x in xrange(x, ex+1):
                action(param, _x, _y, **kwargs)

    def change_current_layer(self, idx):
        self._layer = idx

    def change_tiles_in_a_region(self, tiletype, *vertices, **kwargs):
        if len(vertices) == 2:
            self.change_tile(tiletype, *vertices, **kwargs)
        else:
            self.call_action_in_a_region(self.change_tile, tiletype, *vertices, 
                **kwargs) 

    def change_tile(self, new_tile, x, y, **kwargs):
        if new_tile:
            tile_sprite = tile.Tile(x,y, **kwargs)
            tile_sprite.image = self.resources.indexed_tiles[new_tile][2]
        else:
            tile_sprite = new_tile
        self.tile_map.layers[self._layer].content2D[y][x] = tile_sprite

    def get_tile(self, x, y, layerindex):
        return self.tile_map.layers[layerindex].content2D[y][x]

    def export_map(self):
        return self.tile_map

    def load_tileset(self, filename):
        tileset = tmxreader.TileSet()
        tileset.firstgid = 1
        image = tmxreader.TileImage()
        image.source = resource_manager.get_resource_path(filename)
        tileset.images.append(image)
        self.tile_map.tile_sets.append(tileset)
        self.resources.load(self.tile_map)

    def new_map(self, width, height):
        self._layer = 0
        self.tile_map = tmxreader.TileMap()
        self.tile_map.width = width
        self.tile_map.height = height
        self.tile_map.tilewidth = tile.TILEWIDTH
        self.tile_map.tileheight = tile.TILEHEIGHT
